function logPxy = gmm_logpxy(X,model)
% GMM_LOGPXY Computes logarithm p(x,y) for GMM.
% 
% Synopsis:
%  logPdf = gmm_logpxy(X,model)
%
% Description:
%  Let p(x) be a Gaussian Mixture Model, that is
%
%   p(x) =  sum p(x|y) p(y)
%          y=1:nY
%
%  where p(x|y) are Gaussians components and p(y) discrete probabilities. 
%  This function computes 
% 
%     logPxy(y,i) = log( p(X(:,i),y) ) 
%
% Input:
%  X [ nDim x nVectors] Input n-dimensional real vectors.
%  model [struct] Parameters of the GMM with nY Gaussian components.
%
% Output:
%  logPxy [ nY x nVectors] logPxy(y,i) = log( p(X(:,i),y) ).
%

nY = length(model.Prior);
[nDim,nVectors] = size(X);

logPxy = zeros(nY,nVectors);

switch model.cov_type
  case 'full'
      for y=1:nY
          dist = mahalan(X,model.Mean(:,y),model.Cov(:,:,y));
          logPxy(y,:) = -0.5*(dist + log(det(model.Cov(:,:,y))) + nDim*log(2*pi)) + log(model.Prior(y));
      end  
end

return;
% EOF