function model=gmm_ml(feat,labels,cov_type,biased)
% GMM_ML Maximum Likelihood estimation of Gaussian Mixture model.
% 
% Synopsis:
%  model = gmm_ml(feat,labels)
%  model = gmm_ml(feat,labels,cov_type)
%  model = gmm_ml(feat,labels,cov_type,biased)
% 
% Description:
%  This function computes the Maximum Likelihood estimate of 
%  a joint distribution p(x,y) defined over feature vectors x in Re^n 
%  and labels y in [1:nY] 
%
%     p(x,y) = p(x|y) * p(y) 
%
%  where p(x|y) is Gauss distribution with the mean vector 
%  Mean(:,y) and covariance matrix Cov(:,:,y) (or Cov(:,y) or Cov(y) 
%  depending on covariance shape).
% 
%  The shape of the covariance matrix is controled by the input 
%  argument cov_type 
%   cov_type = 'full'      full covariance matrix (default)
%   cov_type = 'diag'      diagonal covarinace matrix
%   cov_type = 'spherical' spherical covariance matrix
%
%  If biased == 0 (default) and the number of samples N is > 1 then the 
%  covariance matrix is normalized by N-1 which makes it the best unbiased
%  estimate. Otherwise it is normalized by N which corresponds to the ML 
%  estimate.
%
% Input:
%  feat     [nDim x nVectors] Feature vectors.
%  labels   [nDim x nVectors] Labels must be from 1:nY.
%  cov_type [string] Type of covariacne matrix (default 'full').
%  biased   [1x1] If 0 (default) then normalize covariance matrices by N-1.
%
% Output:
%  model [struct] Gaussian model:
%   .Mean  [nDim x nY] Mean vectors.
%   .Cov   [nDim x nDim x nY] Covariance matrices for cov_type = 'full'
%          [nDim x nY] Diagonals of Cov for cov_type = 'diag'
%          [nY x 1] Diagonal element of Cov for cov_type = 'spherical'
%   .Prior [nY x 1] Priory probabilities p(y).
%  
% Example:
%  load('riply_dataset','trn_feat','trn_labels');
%  model = gmm_ml( trn_feat, trn_labels );
%  figure; hold on; ppatterns(data); pgauss( model );
%  figure; hold on; ppatterns(data); pgmm( model );
%
% See also 
%
 
if nargin < 3, 
    cov_type = 'full'; 
end
if nargin < 4,
    biased = 0;
end

[nDim,nVectors] = size(feat);
nY = max(labels);

model.Mean = zeros(nDim,nY);
switch cov_type
    case 'full'
        model.Cov = zeros(nDim,nDim,nY);
    case 'diag' 
        model.Cov = zeros(nDim,nY);
    case 'spherical'
        model.Cov = zeros(nY,1);
end
model.Prior = zeros(nY,1);

for y=1:nY,
   
   idx = find(labels == y);
   n = length(idx);
   
   if n > 0
       model.Mean(:,y) = mean(feat(:,idx),2);
       
       switch cov_type,
         case 'full', 
             model.Cov(:,:,y) = cov(feat(:,idx)',biased);
         case 'diag'
             model.Cov(:,y) = var(feat(:,idx)',biased)';
         case 'spherical',
             model.Cov(y) = var(reshape(feat(:,idx),n*nDim,1),biased);
         otherwise
            error('Unknown cov_type.');
       end
   else
       model.Mean(:,y) = nan;

       switch cov_type,
         case 'full', 
             model.Cov(:,:,y) = nan;
         case 'diag'
             model.Cov(:,y) = nan;
         case 'spherical',
             model.Cov(y) = nan;
       end
   end
   
   model.Prior(y) = n/nVectors;
end

model.cov_type = cov_type;
model.eval = 'pdfgmm';

return; 
% EOF